@@ -33,7 +33,7 @@ ${globalcode}
 ; ---------------------------------
 
 opcode _panweights, kk, k
-    kpos xin    
+    kpos xin   
     kampL = bpf:k(kpos, 0, 1.4142, 0.5, 1, 1, 0)
     kampR = bpf:k(kpos, 0, 0,      0.5, 1, 1, 1.4142)
     xout kampL, kampR
@@ -41,23 +41,19 @@ endop
 
 opcode sfloadonce, i, S
     Spath xin
-    Skey_ strcat "SFLOAD:", Spath
-    iidx dict_get gi__soundfontIndexes, Skey_, -1
+    iidx dict_get gi__soundfontIndexes, Spath, -1
     if (iidx == -1) then
         iidx sfload Spath
-        dict_set gi__soundfontIndexes, Skey_, iidx
+        dict_set gi__soundfontIndexes, Spath, iidx
     endif
     xout iidx
 endop
 
 opcode sfPresetIndex, i, Sii
-    
     Spath, ibank, ipresetnum xin
     isf sfloadonce Spath
     Skey sprintf "SFIDX:%d:%d:%d", isf, ibank, ipresetnum
-    
     iidx dict_get gi__soundfontIndexes, Skey, -1
-        
     if iidx == -1 then
         iidx chnget "_soundfontPresetCount"
         chnset iidx+1, "_soundfontPresetCount"
@@ -151,6 +147,28 @@ instr ${maketable}
     endif
 endin
 
+instr ${automatePargViaPargs}
+    ip1 = p4
+    ipindex = p5
+    imode = p6;  interpolation method
+    iovertake = p7
+    ilenpairs = p8
+    ipairs[] passign 9, 9+ilenpairs
+    iXs[] slicearray ipairs, 0, ilenpairs-1, 2
+    iYs[] slicearray ipairs, 1, ilenpairs-1, 2
+    Sinterpmethod = strget(imode)
+    
+    if iovertake == 1 then
+        icurrval pread ip1, ipindex, -1
+        iYs[0] = icurrval
+    endif
+
+    kt timeinsts
+    kidx bisect kt, iXs
+    ky interp1d kidx, iYs, Sinterpmethod
+    pwrite ip1, ipindex, ky
+endin
+
 instr ${automatePargViaTable}
     ip1 = p4
     ipindex = p5
@@ -379,23 +397,27 @@ instr ${preadmany}
 endin
 
 instr ${testaudio}
-    pset 0, 0, 0, 0
-    imode passign 4
+    pset 0, 0, 0, 0, 1, 1
+    imode = p4
+    iperiod = p5
+    igain = p6
+    
     kchan init -1
     if imode == 0 then
         prints "\nTestaudio: pink noise mode\n"
         a0 pinker
+        a0 *= igain
     elseif imode == 1 then
         prints "\nTestaudio: sine tone mode\n"
-        a0 oscili 0.1, 1000
+        a0 oscili igain, 1000
     else
         initerror sprintf("testaudio: imode %d unknown", imode)
     endif
-    kswitch metro 1
+    kswitch metro 1/iperiod
     kchan = (kchan + kswitch) % nchnls
     outch kchan+1, a0
     if kswitch == 1 then
-        println "Channel: %d", kchan
+        println "Channel: %d / %d", kchan+1, nchnls
     endif
 endin
 
